package provide BTS_USERIO 0.1

# ***********************************************************************
#	Init function: Sets up variables
# ***********************************************************************

namespace eval ::bts::user_io {
	#add te userio config function to base bts
	::bts::bts add_config_function user_io "::bts::user_io::config";
	
	variable bts_userio_running;
	variable bts_userio_wait;
	set bts_userio_running 0;
	
	variable bts_userio;
	#Setup supported userio types
	#if they are set to 0, then they are disabled
	#1 for each digit [size] - uses nr_pio_showhex which assumes peripheral name of sev seg display
	set bts_userio(seven_segment_display) 0;
	#1 per LED [size write_address_on_SOPC_system]
	set bts_userio(leds) 0;
	#i per PB [size read_address_on_SOPC_system]
	set bts_userio(pushbuttons) 0;
	#1 per dipswitch position [size read_address_on_SOPC_system]
	set bts_userio(dipswitch) 0;
	
}

proc ::bts::user_io::config {instruction var2 var3 var4 var5 var6} {
	variable bts_userio;
	if {$instruction == ""} {
		set insruction "help";
	}
	
	switch -exact -- $instruction {
		help {
			::bts::bts_base_print std "To be added!\n";
		}
		status {
			::bts::bts_base_print std "Current User IO Settings are:\n";
			foreach index [array names bts_userio] {
				::bts::bts_base_print std "$index : $bts_userio($index)\n";
			}
		}
		set {
			if {[regexp $var2 [array names bts_userio]]} {
				set bts_userio($var2) $var3;
			} else {
				::bts::bts_base_print error "$var2 is nat a valid user_io assignment.\n";
			}
		}
		build {
			return [::bts::user_io::build $var3 $var4 $var5];
		}
		default {
			::bts::bts_base_print error "$var2 is an invalid user io instruction\n";
		}
	}
}

#build the userio page
proc ::bts::user_io::build {notebook tab frame} {
	variable bts_userio;
	variable bts_userio_running;
	
	#set raise and lower functions
	$notebook itemconfigure $tab -raisecmd "::bts::user_io::tab_raised_function";
	$notebook itemconfigure $tab -leavecmd "::bts::user_io::tab_lowered_function";
	
	set userio_frame [frame $frame.userio_frame];
	
	#frame for seg and dipsw
	set seg_and_dip_frame [frame $userio_frame.seg_and_dip_frame];
	#seven_seg disp
	if {[lindex $bts_userio(seven_segment_display) 0]} {
		#make frame and label
		set sev_seg_frame [frame $seg_and_dip_frame.sev_seg_frame -borderwidth 3 -relief sunken];
		set seg_label [label $sev_seg_frame.seg_label -text "Seven Segment Display"];
		
		pack $seg_label -in $sev_seg_frame -side top -padx 3 -pady 3;
		set seven_seg_scale [scale $sev_seg_frame.ss_scale -from 0 -to 99 \
				-showvalue true -length 120 -orient horizontal \
				-command {::bts::user_io::sev_seg::update ::bts::user_io::bts_userio(seven_segment_display)}];
		pack $seven_seg_scale -in $sev_seg_frame;
		
		set digit_canvas [canvas $sev_seg_frame.canvas  -height 50 -width 60 -relief sunken]
		set ::bts::user_io::sev_seg::bts_sev_seg $seven_seg_scale;
		set ::bts::user_io::sev_seg::bts_sev_seg_canvas $digit_canvas;
		pack $digit_canvas -in $sev_seg_frame;
		
		pack $sev_seg_frame -side top -padx 10 -pady 10;
	}
	
	#dipswitch
	if {[lindex $bts_userio(dipswitch) 0]} {
		
		#create the frame
		set dip_frame [frame $seg_and_dip_frame.dip_frame -borderwidth 3 -relief sunken];
		#label it
		#set dip_label [label $dip_frame.dip_label -text "Dipswitch"];
		#pack $dip_label -in $dip_frame -anchor nw -padx 10 -pady 10;
		
		#count the number of dipswitches
		set dipsw_num 0;
		foreach {dip_size dip_address} $bts_userio(dipswitch) {			
			set ::bts::user_io::dipswitch::bts_dip(dip_size_$dipsw_num) $dip_size;
			set ::bts::user_io::dipswitch::bts_dip(dip_address_$dipsw_num) $dip_address;
			incr dipsw_num;			
		}
		
		set ::bts::user_io::dipswitch::bts_dip(number_of_dipswitches) $dipsw_num;
		
		for {set x 0} {$x < $dipsw_num} {incr x} {
			#label the dipswitch
			pack [label $dip_frame.label$x -text "Dipswitch $x"] -anchor nw;
			
			#get the number of dipswitch positions
			set dip_size $::bts::user_io::dipswitch::bts_dip(dip_size_$x);
			
			#build the canvas
			set canvas [canvas $dip_frame.canvas_$x \
					-height 50 -width [expr 10+$dip_size*22] \
					-borderwidth 3 -relief ridge];
			set ::bts::user_io::dipswitch::bts_dip(canvas_$x) $canvas;

			for {set xx 0} {$xx < $dip_size} {incr xx} {
				set seg_name "_$xx";
				set seg_name "$x$seg_name";
				set ::bts::user_io::dipswitch::bts_dip(dip_$seg_name) [$canvas create image [expr 10+22*$xx] 10 \
						-image $::bts::user_io::dipswitch::bts_dip(off) -anchor nw]
			}
			pack $canvas;
			
		}
		pack $dip_frame -side top -padx 10 -pady 10;
		
		
	}
	
	#frame for LED and PBs
	set led_and_pb_frame [frame $userio_frame.led_and_pb_frame];
	#build the LEDs
	if {[lindex $bts_userio(leds) 0]} {
		#make led_frame and fill it

		set led_frame [frame $led_and_pb_frame.led_frame -borderwidth 3 -relief sunken];
		set led_label [label $led_frame.led_label -text "LEDs"];
		
		#create buttons
		set led_button_on [button $led_frame.ledon -command {::bts::user_io::leds::leds_on $::bts::user_io::bts_userio(leds)} -text "LEDs On" ];
		set led_button_off [button $led_frame.ledoff -command {::bts::user_io::leds::leds_off $::bts::user_io::bts_userio(leds)} -text "LEDs Off"];
		
		#put the buttons in
		pack $led_label -in $led_frame -side top -padx 3 -pady 3;
		pack $led_button_on -in $led_frame ;
		pack $led_button_off -in $led_frame;
		
		#create LED Check boxes
		for {set x 0} {$x < [lindex $bts_userio(leds) 0]} {incr x} {
		    
		    checkbutton $led_frame.led$x -onvalue 0 -offvalue [expr pow(2,$x)] -variable ::bts::user_io::leds::bts_leds(led$x) -command {::bts::user_io::leds::update [lindex $::bts::user_io::bts_userio(leds) 1] } -text "LED $x";
		    pack $led_frame.led$x -in $led_frame;
		}
		pack $led_frame -side left -padx 10 -pady 10;
				
	}
	
	
	if {[lindex $bts_userio(pushbuttons) 0]} {
		set ::bts::user_io::pushbuttons::bts_pb(pb_list) $bts_userio(pushbuttons);
		
		set pb_f [frame $led_and_pb_frame.pb_f -borderwidth 3 -relief sunken];
		set pb_label [label $pb_f.label -text "PushButtons"];
		pack $pb_label -anchor n -padx 10 -pady 10;
		set num_buttons [lindex $bts_userio(pushbuttons) 0];
		#set pb_canvas [canvas $pb_f.canvas -height 40 -width [expr [expr 30 * $num_buttons] + 20] ];
		set pb_canvas [canvas $pb_f.canvas -height [expr [expr 30 * $num_buttons] + 20] -width 50];
		set ::bts::user_io::pushbuttons::bts_pb(canvas) $pb_canvas;
		
		# # Put images onto canvas
		for {set x 0} {$x<$num_buttons} {incr x} {
		    set ::bts::user_io::pushbuttons::bts_pb(button$x) [$pb_canvas create image 10 [expr 10+$x*30] -image $::bts::user_io::pushbuttons::bts_pb(released) -anchor nw]
		}
		pack $pb_canvas;
		pack $pb_f -side left -padx 10 -pady 10;
	}
	set control_frame [frame $userio_frame.control_frame];
	pack [button $control_frame.on_button -command "::bts::user_io::poll_function" -text "Enable Polling"] -side top -anchor nw;
	
	pack $control_frame $seg_and_dip_frame $led_and_pb_frame -side left -anchor n
	pack $userio_frame -in $frame;
	
}

#bts userio raise function

proc ::bts::user_io::tab_raised_function {} {
	variable bts_userio;
	#update the seven segment display
	if {[lindex $bts_userio(seven_segment_display) 0]} {
		#::bts::user_io::sev_seg::update $::bts::user_io::bts_userio(seven_segment_display);
		$::bts::user_io::sev_seg::bts_sev_seg set 0;
	}
	if {[lindex $bts_userio(leds) 0]} {
		::bts::user_io::leds::update [lindex $::bts::user_io::bts_userio(leds) 1];
	}
	
}


#checks PBs DIPs value
proc ::bts::user_io::poll_function {} {
	variable bts_userio_running;
	variable bts_userio;
	variable bts_userio_wait;
	#tab has been raised
	set bts_userio_running 1;
	
	set command_id_pb [::bts::bts comm set_cont_function_id "::bts::user_io::pushbuttons::update"];
	set command_id_dp [::bts::bts comm set_cont_function_id "::bts::user_io::dipswitch::update"];
	
	while {$bts_userio_running} {
		set bts_userio_wait 0;
		after 100 {set bts_userio_wait 1};
		vwait bts_userio_wait;
		if {[lindex $bts_userio(pushbuttons) 0]} {
			::bts::bts comm function_id_write_port $command_id_pb 4 1 [list [lindex $bts_userio(pushbuttons) 1]];
		}
		
		set dip_address_list "";
		
		if {[lindex $bts_userio(dipswitch) 0]} {
			set number_of_dips $::bts::user_io::dipswitch::bts_dip(number_of_dipswitches);
			

			for {set x 0} {$x < $number_of_dips} {incr x} {
				set address $::bts::user_io::dipswitch::bts_dip(dip_address_$x);

				if {$x == 0} {
					set address_list $address;
				} else {
					lappend address_list $address;
				}				
			}

			::bts::bts comm function_id_write_port $command_id_dp 4 $number_of_dips $address_list;
		}
	}

	#must unset the command_id's but wait for a bit to ensure that all 
	#data requested has been returned.
	set bts_userio_wait 0;
	after 1000 {set bts_userio_wait 1};
	vwait bts_userio_wait;
	::bts::bts comm unset_cont_function_id $command_id_dp;
	::bts::bts comm unset_cont_function_id $command_id_pb;
	
}

#bts userio lower function
#stops checking of PBs DIPs value
proc ::bts::user_io::tab_lowered_function {} {
	variable bts_userio_running;
	set bts_userio_running 0;
	return 1;
}

#led stuff
namespace eval ::bts::user_io::leds {	
	variable bts_leds;
}

proc ::bts::user_io::leds::update {address} {
	variable bts_leds;
	
	set led_value 0;
	
	foreach led [array names bts_leds] {
		set led_value [expr $led_value + $bts_leds($led)];
	}
	set led_value [format "%.0f" $led_value];
	set led_write_value [format "%x" $led_value];
	::bts::bts comm write_port 1 1 [list $address $led_write_value];
}

proc ::bts::user_io::leds::leds_on {leds_list} {
	variable bts_leds;
	foreach led [array names bts_leds] {
		set bts_leds($led) 0;		
	}
	::bts::user_io::leds::update [lindex $leds_list 1];
}
proc ::bts::user_io::leds::leds_off {leds_list} {
	variable bts_leds;
	for {set x 0} {$x < [lindex $leds_list 0]} {incr x} {
		set bts_leds(led$x) [expr pow(2,$x)];
	}
	::bts::user_io::leds::update [lindex $leds_list 1];
}



#pb stuff
namespace eval ::bts::user_io::pushbuttons {	
	variable bts_pb;
	set bts_pb(pressed) [image create photo -file "./BTS_BASE/pics/bdbutton_pressed.gif"]
	set bts_pb(released) [image create photo -file "./BTS_BASE/pics/bdbutton.gif"]
	
}
proc ::bts::user_io::pushbuttons::update {number_list} {
	variable bts_pb;
	
	set number [lindex $number_list 0];
	set number [format "%u" "0x$number"];
	for {set x 0} {$x < [lindex $bts_pb(pb_list) 0]} {incr x} {
		if {($number & [format "%.0f" [expr pow(2,$x)]]) == 0} {
			$bts_pb(canvas) itemconfigure $bts_pb(button$x) -image $bts_pb(pressed);
		} else {
			$bts_pb(canvas) itemconfigure $bts_pb(button$x) -image $bts_pb(released);
		}
	}
	
	#This return is always complete after 1 call.  therefore always return 1
	#to unset the return pointer in the bts comm functions
	return 1;
	
}
#dip stuff
namespace eval ::bts::user_io::dipswitch {	
	variable bts_dip;
	set bts_dip(on) [image create photo -file "./BTS_BASE/pics/dip_sw_on.gif"];
	set bts_dip(off) [image create photo -file "./BTS_BASE/pics/dip_sw_off.gif"];
}

proc ::bts::user_io::dipswitch::update {number_list} {
	variable bts_dip;
	set number_of_dipsw $bts_dip(number_of_dipswitches);
	
	for {set x 0} {$x < $number_of_dipsw} {incr x} {
		set number [lindex $number_list $x];
		set number [format "%u" "0x$number"];
		for {set xx 0} {$xx < $bts_dip(dip_size_$x)} {incr xx} {
			set seg_name $x;
			set seg_name2 "_$xx";
			set seg_name "$seg_name$seg_name2";
			if {($number & [format "%.0f" [expr pow(2,$xx)]]) == 0} {
				 $bts_dip(canvas_$x) itemconfigure $bts_dip(dip_$seg_name) -image $bts_dip(on);
			 } else {
				 $bts_dip(canvas_$x) itemconfigure $bts_dip(dip_$seg_name) -image $bts_dip(off);
			 }
			
		}
	}
	
	#This return is always complete after 1 call.  therefore always return 1
	#to unset the return pointer in the bts comm functions
	return 1;
	
}

#seven_segment_stuff
namespace eval ::bts::user_io::sev_seg {	
	variable bts_sev_seg_canvas;
	variable bts_sev_seg;
}
proc ::bts::user_io::sev_seg::update {seg_list number} {
	variable bts_sev_seg_canvas;
	
	::bts::bts comm write_port 5 1 $number;
	showLCD $bts_sev_seg_canvas $number 2;
}


#this snippit of code was found here : http://aspn.activestate.com/ASPN/Cookbook/Tcl/Recipe/68382
#and was available with this license
# License
# 
# The source code in this recipe is in the public domain.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ''AS IS'' 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
# IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.

namespace eval ::bts::user_io::sev_seg {
	# The shapes of individual elements of a digit.  They were worked out
	# with a piece of paper, a calculator and a good deal of head-scratching.
	array set lcdshape {
	   a {3.0 5 5.2 3 7.0 5 6.0 15 3.8 17 2.0 15}
	   b {6.3 2 8.5 0 18.5 0 20.3 2 18.1 4 8.1 4}
	   c {19.0 5 21.2 3 23.0 5 22.0 15 19.8 17 18.0 15}
	   d {17.4 21 19.6 19 21.4 21 20.4 31 18.2 33 16.4 31}
	   e {3.1 34 5.3 32 15.3 32 17.1 34 14.9 36 4.9 36}
	   f {1.4 21 3.6 19 5.4 21 4.4 31 2.2 33 0.4 31}
	   g {4.7 18 6.9 16 16.9 16 18.7 18 16.5 20 6.5 20}
	}
	# Which elements are turned on for a given digit?  Note that this is only
	# decimal digits, though adding hex digits should be easy enough.
	array set llcd {
	   0 {a b c d e f}
	   1 {c d}
	   2 {b c e f g}
	   3 {b c d e g}
	   4 {a c d g}
	   5 {a b d e g}
	   6 {a b d e f g}
	   7 {b c d}
	   8 {a b c d e f g}
	   9 {a b c d e g}
	   - {g}
	   { } {}
	}
	# Which elements are turned off for a given digit?  Note that you should make
	# sure that this is the complement of the digits set in the llcd array...
	array set ulcd {
	   0 {g}
	   1 {a b e f g}
	   2 {a d}
	   3 {a f}
	   4 {b e f}
	   5 {c f}
	   6 {c}
	   7 {a e f g}
	   8 {}
	   9 {f}
	   - {a b c d e f}
	   { } {a b c d e f g}
	}
}


# Displays a decimal number using LCD digits in the top-left of the canvas
proc bts::user_io::sev_seg::showLCD {
   canvas number {width 2} {colours {#ff8080 #ff0000 grey grey}}
} {
   variable llcd;
   variable ulcd ;
   variable lcdshape;
   
   set lcdoffset 0

   # Remove previous LCD number
   $canvas delete lcd

   # Get colours to draw with
   foreach {onRim onFill offRim offFill} $colours {break}

   # For each digit in the (space-padded) number...
   foreach glyph [split [format %0${width}d $number] {}] {
      # ... draw its "on" elements...
      foreach symbol $llcd($glyph) {
         # ... by drawing a polygon and shifting it...
         $canvas move [eval $canvas create polygon $lcdshape($symbol) -tags lcd -outline $onRim -fill $onFill] $lcdoffset 0
      }
      # ... and then doing the same thing for the "off" elements
      foreach symbol $ulcd($glyph) {
         $canvas move [eval $canvas create polygon $lcdshape($symbol) -tags lcd -outline $offRim -fill $offFill] $lcdoffset 0
      }
      # And now we increase our offset too...
      incr lcdoffset 22
   }
}


